@@ -1,11 +1,11 @@
 package SVN::Notify;
 
-# $Id: Notify.pm 2778 2006-04-04 23:07:47Z theory $
+# $Id: Notify.pm 2836 2006-05-05 20:42:04Z theory $
 
 use strict;
 use constant WIN32  => $^O eq 'MSWin32';
 use constant PERL58 => $] > 5.007;
-$SVN::Notify::VERSION = '2.56';
+$SVN::Notify::VERSION = '2.58';
 
 =begin comment
 
@@ -44,7 +44,7 @@ Use the class in a custom script:
 
 This class may be used for sending email messages for Subversion repository
 activity. There are a number of different modes supported, and SVN::Notify is
-fully subclassable, to easily add new functionality. By default, A list of all
+fully subclassable to easily add new functionality. By default, A list of all
 the files affected by the commit will be assembled and listed in a single
 message. An additional option allows diffs to be calculated for the changes
 and either appended to the message or added as an attachment. See the
@@ -62,10 +62,12 @@ everything executes properly.
 =cut
 
 # Map the svnlook changed codes to nice labels.
-my %map = ( U => 'Modified Paths',
-            A => 'Added Paths',
-            D => 'Removed Paths',
-            _ => 'Property Changed');
+my %map = (
+    U => 'Modified Paths',
+    A => 'Added Paths',
+    D => 'Removed Paths',
+    _ => 'Property Changed',
+);
 
 ##############################################################################
 
@@ -169,7 +171,7 @@ domain specified by the C<user_domain> parameter.
 =item svnlook
 
   svnnotify --svnlook /path/to/svnlook
-  svnnotify --l /path/to/svnlook
+  svnnotify -l /path/to/svnlook
 
 The location of the F<svnlook> executable. If not specified, SVN::Notify will
 search through the directories in the C<$PATH> environment variable, plus in
@@ -188,16 +190,49 @@ executable.
 =item sendmail
 
   svnnotify --sendmail /path/to/sendmail
-  svnnotify --s /path/to/sendmail
+  svnnotify -s /path/to/sendmail
 
-The location of the F<sendmail> executable. If not specified, SVN::Notify will
-search through the directories in the C<$PATH> environment variable, plus in
-F</usr/local/bin> and F</usr/sbin>, for an F<sendmail> executable. Specify a
-full path to F<sendmail> via this option or by setting the C<$SENDMAIL>
-environment variable if F<sendmail> isn't in your path or to avoid loading
+The location of the F<sendmail> executable. If neither the C<sendmail> nor the
+C<smtp> parameter is specified, SVN::Notify will search through the
+directories in the C<$PATH> environment variable, plus in F</usr/local/bin>
+and F</usr/sbin>, for an F<sendmail> executable. Specify a full path to
+F<sendmail> via this option or by setting the C<$SENDMAIL> environment
+variable if F<sendmail> isn't in your path or to avoid loading
 L<File::Spec|File::Spec>. The same caveats as applied to the location of the
 F<svnlook> executable apply here.
 
+=item smtp
+
+  svnnotify --smtp smtp.example.com
+
+The address for an SMTP server through which to send the notification email.
+If unspecified, SVN::Notify will use F<sendmail> to send the message. If
+F<sendmail> is not installed locally (such as on Windows boxes!), you I<must>
+specify an SMTP server.
+
+=item smtp_user
+
+  svnnotify --smtp-user myuser
+
+The user name for SMTP authentication. If this option is specified,
+SVN::Notify will use L<Net::SMTP_auth|Net::SMTP_auth> to send the notification
+message, and will of course authenticate to the SMTP server.
+
+=item smtp_pass
+
+  svnnotify --smtp-pass mypassword
+
+The password for SMTP authentication. Use in parallel with C<smtp_user>.
+
+=item smtp_authtype
+
+  svnnotify --smtp-authtype authtype
+
+The authentication method to use for authenticating to the SMTP server. The
+available athentication types include "PLAIN", "NTLM", "CRAM_MD5", and others.
+Consult the L<Authen::SASL|Authen::SASL> documentation for a complete list.
+Defaults to "PLAIN".
+
 =item charset
 
   svnnotify --charset UTF-8
@@ -253,7 +288,7 @@ implicitly sets the C<with_diff> parameter to a true value.
 =item reply_to
 
   svnnotify --reply-to devlist@example.com
-  svnnotify --R developers@example.net
+  svnnotify -R developers@example.net
 
 The email address to use in the "Reply-To" header of the notification email.
 No "Reply-To" header will be added to the email if no value is specified for
@@ -295,7 +330,7 @@ Pass an array reference if calling C<new()> directly.
 =item no_first_line
 
   svnnotify --no-first-line
-  svnnotify --O
+  svnnotify -O
 
 Omits the first line of the log message from the subject. This is most useful
 when used in combination with the C<subject_cx> parameter, so that just the
@@ -426,7 +461,9 @@ ticket system. The URL must have the "%s" format where the first match
 The regex to match a ticket tag of a custom ticket system. This should return
 a single match to be interpolated into the C<ticket_url> option. The example
 shown matches "[#1234]" or "#1234" or "[# 1234]". This regex should be as
-specific as possible, preferably wrapped in "\b" tags and the like.
+specific as possible, preferably wrapped in "\b" to match word boundaries. If
+you're using L<SVN::Notify::HTML|SVN::Notify::HTML>, be sure to read its
+documentation for a different syntax for C<ticket_regex>!
 
 =item verbose
 
@@ -476,12 +513,14 @@ sub new {
       unless $params{to} || $params{to_regex_map};
 
     # Set up default values.
-    $params{svnlook}   ||= $ENV{SVNLOOK}  || $class->find_exe('svnlook');
-    $params{sendmail}  ||= $ENV{SENDMAIL} || $class->find_exe('sendmail');
-    $params{with_diff} ||= $params{attach_diff};
-    $params{verbose}   ||= 0;
-    $params{charset}   ||= 'UTF-8';
-    $params{io_layer}  ||= "encoding($params{charset})";
+    $params{svnlook}        ||= $ENV{SVNLOOK}  || $class->find_exe('svnlook');
+    $params{with_diff}      ||= $params{attach_diff};
+    $params{verbose}        ||= 0;
+    $params{charset}        ||= 'UTF-8';
+    $params{io_layer}       ||= "encoding($params{charset})";
+    $params{smtp_authtype}  ||= 'PLAIN';
+    $params{sendmail}       ||= $ENV{SENDMAIL} || $class->find_exe('sendmail')
+        unless $params{smtp};
 
     # svnweb_url and viewcvs_url are mutually exlusive.
     if ($params{svnweb_url} && $params{svnweb_url} !~ /%s/) {
@@ -585,6 +624,7 @@ sub get_options {
         'user-domain|D=s'     => \$opts->{user_domain},
         'svnlook|l=s'         => \$opts->{svnlook},
         'sendmail|s=s'        => \$opts->{sendmail},
+        'smtp=s'              => \$opts->{smtp},
         'charset|c=s'         => \$opts->{charset},
         'io-layer|o=s'        => \$opts->{io_layer},
         'language|g=s'        => \$opts->{language},
@@ -612,6 +652,9 @@ sub get_options {
         'version|v'           => \$opts->{version},
         'header=s'            => \$opts->{header},
         'footer=s'            => \$opts->{footer},
+        'smtp-user=s'         => \$opts->{smtp_user},
+        'smtp-pass=s'         => \$opts->{smtp_pass},
+        'smtp-authtype=s'     => \$opts->{smtp_authtype},
     ) or return;
 
     # Load a subclass if one has been specified.
@@ -926,9 +969,9 @@ sub prepare_subject {
   $notifier->execute;
 
 Sends the notification message. This involves opening a file handle to
-F<sendmail> and passing it to C<output()>. This is the main method used to
-send notifications or execute any other actions in response to Subversion
-activity.
+F<sendmail> or a tied file handle connected to an SMTP server and passing it
+to C<output()>. This is the main method used to send notifications or execute
+any other actions in response to Subversion activity.
 
 =cut
 
@@ -937,8 +980,9 @@ sub execute {
     $self->_dbpnt( "Sending message") if $self->{verbose};
     return $self unless $self->{to};
 
-    # Safe pipe to sendmail. See perlipc(1).
-    my $out = $self->_pipe('|-', $self->{sendmail}, '-oi', '-t');
+    my $out = $self->{smtp}
+        ? SVN::Notify::SMTP->get_handle($self)
+        : $self->_pipe('|-', $self->{sendmail}, '-oi', '-t');
 
     # Output the message.
     $self->output($out);
@@ -956,9 +1000,9 @@ sub execute {
   $notifier->output($file_handle, $no_headers);
 
 Called internally by C<execute()> to output a complete email message. The file
-handle passed is a piped connection to F<sendmail>, so that C<output()> and
-its related methods can print directly to sendmail. The optional second
-argument, if true, will suppress the output of the email headers.
+a file handle, so that C<output()> and its related methods can print directly
+to the email message. The optional second argument, if true, will suppress the
+output of the email headers.
 
 Really C<output()> is a simple wrapper around a number of other method calls.
 It is thus essentially a shortcut for:
@@ -1392,6 +1436,7 @@ __PACKAGE__->_accessors(qw(
     user_domain
     svnlook
     sendmail
+    smtp
     charset
     io_layer
     language
@@ -1500,6 +1545,13 @@ Gets or sets the value of the C<svnlook> attribute.
 
 Gets or sets the value of the C<sendmail> attribute.
 
+=head3 smtp
+
+  my $smtp = $notifier->smtp;
+  $notifier = $notifier->smtp($smtp);
+
+Gets or sets the value of the C<smtp> attribute.
+
 =head3 charset
 
   my $charset = $notifier->charset;
@@ -1679,8 +1731,12 @@ sub _pipe {
     $self->_dbpnt( "Piping execution of '" . join("', '", @_) . "'")
       if $self->{verbose};
     # Safer version of backtick (see perlipc(1)).
-    # XXX Use Win32::Process on Win32? This doesn't seem to work as-is on Win32.
     local *PIPE;
+    if (WIN32) {
+        my $cmd = $mode eq '-|' ? join(q{ }, @_) . '|' : '|' . join q{ }, @_;
+        open PIPE, $cmd or die "Cannot fork: $!\n";
+        return *PIPE;
+    }
     my $pid = open PIPE, $mode;
     die "Cannot fork: $!\n" unless defined $pid;
 
@@ -1716,6 +1772,52 @@ sub _read_pipe {
 
 sub _dbpnt { print __PACKAGE__, ": $_[1]\n" }
 
+package SVN::Notify::SMTP;
+
+sub get_handle {
+    my ($class, $notifier) = @_;
+
+    # Load Net::SMTP or the appropriate subclass.
+    my $smtp_class = do {
+        if ($notifier->{smtp_user}) {
+            require Net::SMTP_auth;
+            'Net::SMTP_auth';
+        } else {
+            require Net::SMTP;
+            'Net::SMTP';
+        }
+    };
+
+    my $smtp = $smtp_class->new(
+        $notifier->{smtp},
+        ( $notifier->{verbose} > 1 ? ( Debug => 1 ) : ())
+    );
+
+    $smtp->auth( @{ $notifier }{qw(smtp_authtype smtp_user smtp_pass)} )
+        if $notifier->{smtp_user};
+
+    binmode tied(*{ $smtp->tied_fh }), ":$notifier->{io_layer}"
+        if SVN::Notify::PERL58;
+    $smtp->mail($notifier->{from});
+    $smtp->to($notifier->{to});
+    $smtp->data;
+    tie local(*SMTP), $class, $smtp;
+    return *SMTP;
+}
+
+sub TIEHANDLE {
+    my ($class, $smtp) = @_;
+    bless \$smtp, $class;
+}
+
+sub PRINT  { ${+shift}->datasend(@_) }
+sub PRINTF { ${+shift}->datasend(sprintf shift, @_) }
+sub CLOSE  {
+    my $self = shift;
+    $$self->dataend;
+    $$self->quit;
+}
+
 1;
 __END__
 
@@ -1739,18 +1841,6 @@ SourceForge.net support for SVN::Notify.
 
 Please send bug reports to <bug-svn-notify@rt.cpan.org>.
 
-=head1 To Do
-
-=over
-
-=item *
-
-Port to Win32. I think it just needs to use Win32::Process to manage
-communication with F<svnlook> and F<sendmail>. See comments in the source
-code.
-
-=back
-
 =head1 Author
 
 =begin comment
